==> cytoscape-2.2.0.tgz: package/src/extensions/layout.arbor.js <==
;(function($$){ 'use strict';
  
  var defaults = {
    liveUpdate: true, // whether to show the layout as it's running
    ready: undefined, // callback on layoutready 
    stop: undefined, // callback on layoutstop
    maxSimulationTime: 4000, // max length in ms to run the layout
    fit: true, // reset viewport to fit default simulationBounds
    padding: [ 50, 50, 50, 50 ], // top, right, bottom, left
    simulationBounds: undefined, // [x1, y1, x2, y2]; [0, 0, width, height] by default
    ungrabifyWhileSimulating: true, // so you can't drag nodes during layout

    // forces used by arbor (use arbor default on undefined)
    repulsion: undefined,
    stiffness: undefined,
    friction: undefined,
    gravity: true,
    fps: undefined,
    precision: undefined,

    // static numbers or functions that dynamically return what these
    // values should be for each element
    nodeMass: undefined, 
    edgeLength: undefined,

    stepSize: 1, // size of timestep in simulation

    // function that returns true if the system is stable to indicate
    // that the layout can be stopped
    stableEnergy: function( energy ){
      var e = energy; 
      return (e.max <= 0.5) || (e.mean <= 0.3);
    }
  };
  
  function ArborLayout(options){
    this.options = $$.util.extend({}, defaults, options);
  }
    
  ArborLayout.prototype.run = function(){
    var options = this.options;
    var cy = options.cy;
    var nodes = cy.nodes();
    var edges = cy.edges();
    var container = cy.container();
    var width = container.clientWidth;
    var height = container.clientHeight;
    var simulationBounds = options.simulationBounds;

    if( options.simulationBounds ){
      width = simulationBounds[2] -  simulationBounds[0]; // x2 - x1
      height = simulationBounds[3] - simulationBounds[1]; // y2 - y1
    } else {
      options.simulationBounds = [
        0,
        0, 
        width,
        height
      ];
    }

    // make nice x & y fields
    var simBB = options.simulationBounds;
    simBB.x1 = simBB[0];
    simBB.y1 = simBB[1];
    simBB.x2 = simBB[2];
    simBB.y2 = simBB[3];

    // arbor doesn't work with just 1 node 
    if( cy.nodes().size() <= 1 ){
      if( options.fit ){
        cy.reset();
      }

      cy.nodes().position({
        x: Math.round( (simBB.x1 + simBB.x2)/2 ),
        y: Math.round( (simBB.y1 + simBB.y2)/2 )
      });

      cy.one('layoutready', options.ready);
      cy.trigger('layoutready');

      cy.one('layoutstop', options.stop);
      cy.trigger('layoutstop');

      return;
    }

    var sys = this.system = arbor.ParticleSystem(options.repulsion, options.stiffness, options.friction, options.gravity, options.fps, options.dt, options.precision);
    this.system = sys;

    if( options.liveUpdate && options.fit ){
      cy.reset();
    };
    
    var doneTime = 250;
    var doneTimeout;
    
    var ready = false;
    
    var lastDraw = +new Date;
    var sysRenderer = {
      init: function(system){
      },
      redraw: function(){
        var energy = sys.energy();

        // if we're stable (according to the client), we're done
        if( options.stableEnergy != null && energy != null && energy.n > 0 && options.stableEnergy(energy) ){
          sys.stop();
          return;
        }

        clearTimeout(doneTimeout);
        doneTimeout = setTimeout(doneHandler, doneTime);
        
        var movedNodes = [];
        
        sys.eachNode(function(n, point){ 
          var id = n.name;
          var data = n.data;
          var node = data.element;
          
          if( node == null ){
            return;
          }
          
          if( !node.locked() && !node.grabbed() ){
            node.silentPosition({
              x: simBB.x1 + point.x,
              y: simBB.y1 + point.y
            });

            movedNodes.push( node );
          }
        });
        

        var timeToDraw = (+new Date - lastDraw) >= 16;
        if( options.liveUpdate && movedNodes.length > 0 && timeToDraw ){
          new $$.Collection(cy, movedNodes).rtrigger('position');
          lastDraw = +new Date;
        }

        
        if( !ready ){
          ready = true;
          cy.one('layoutready', options.ready);
          cy.trigger('layoutready');
        }
      }
      
    };
    sys.renderer = sysRenderer;
    sys.screenSize( width, height );
    sys.screenPadding( options.padding[0], options.padding[1], options.padding[2], options.padding[3] );
    sys.screenStep( options.stepSize );

    function calculateValueForElement(element, value){
      if( value == null ){
        return undefined;
      } else if( typeof value == typeof function(){} ){
        return value.apply(element, [element._private.data, {
          nodes: nodes.length,
          edges: edges.length,
          element: element
        }]); 
      } else {
        return value;
      }
    }
    
    // TODO we're using a hack; sys.toScreen should work :(
    function fromScreen(pos){
      var x = pos.x;
      var y = pos.y;
      var w = width;
      var h = height;
      
      var left = -2;
      var right = 2;
      var top = -2;
      var bottom = 2;
      
      var d = 4;
      
      return {
        x: x/w * d + left,
        y: y/h * d + right
      };
    }
    
    var grabHandler = function(e){
      grabbed = this;
      var pos = sys.fromScreen( this.position() );
      var p = arbor.Point(pos.x, pos.y);
      this.scratch().arbor.p = p;
      
      switch( e.type ){
      case 'grab':
        this.scratch().arbor.fixed = true;
        break;
      case 'dragstop':
        this.scratch().arbor.fixed = false;
        this.scratch().arbor.tempMass = 1000
        break;
      }
    };
    nodes.bind('grab drag dragstop', grabHandler);
          
    nodes.each(function(i, node){
      if( this.isFullAutoParent() ){ return; } // they don't exist in the sim

      var id = this._private.data.id;
      var mass = calculateValueForElement(this, options.nodeMass);
      var locked = this._private.locked;

      if( node.isFullAutoParent() ){
        return;
      }
      
      var pos = fromScreen({
        x: node.position().x,
        y: node.position().y
      });

      if( node.locked() ){
        return;
      }

      this.scratch().arbor = sys.addNode(id, {
        element: this,
        mass: mass,
        fixed: locked,
        x: locked ? pos.x : undefined,
        y: locked ? pos.y : undefined
      });
    });
    
    edges.each(function(){
      var id = this.id();
      var src = this.source().id();
      var tgt = this.target().id();
      var length = calculateValueForElement(this, options.edgeLength);
      
      this.scratch().arbor = sys.addEdge(src, tgt, {
        length: length
      });
    });
    
    function packToCenter(callback){
      // TODO implement this for IE :(
      
      if( options.fit ){
        cy.fit();
      }
      callback();
    };
    
    var grabbableNodes = nodes.filter(":grabbable");
    // disable grabbing if so set
    if( options.ungrabifyWhileSimulating ){
      grabbableNodes.ungrabify();
    }
    
    var doneHandler = function(){
      if( window.isIE ){
        packToCenter(function(){
          done();
        });
      } else {
        done();
      }
      
      function done(){
        if( !options.liveUpdate ){
          if( options.fit ){
            cy.reset();
          }

          cy.nodes().rtrigger('position');
        }

        // unbind handlers
        nodes.unbind('grab drag dragstop', grabHandler);
        
        // enable back grabbing if so set
        if( options.ungrabifyWhileSimulating ){
          grabbableNodes.grabify();
        }

        cy.one('layoutstop', options.stop);
        cy.trigger('layoutstop');
      }
    };
    
    sys.start();
    setTimeout(function(){
      sys.stop();
    }, options.maxSimulationTime);
    
  };

  ArborLayout.prototype.stop = function(){
    if( this.system != null ){
      system.stop();
    }
  };
  
  $$('layout', 'arbor', ArborLayout);
  
  
})(cytoscape);

==> cytoscape-2.2.6.tgz: package/src/extensions/layout.arbor.js <==
;(function($$){ 'use strict';
  
  var defaults = {
    liveUpdate: true, // whether to show the layout as it's running
    ready: undefined, // callback on layoutready 
    stop: undefined, // callback on layoutstop
    maxSimulationTime: 4000, // max length in ms to run the layout
    fit: true, // reset viewport to fit default simulationBounds
    padding: [ 50, 50, 50, 50 ], // top, right, bottom, left
    simulationBounds: undefined, // [x1, y1, x2, y2]; [0, 0, width, height] by default
    ungrabifyWhileSimulating: true, // so you can't drag nodes during layout

    // forces used by arbor (use arbor default on undefined)
    repulsion: undefined,
    stiffness: undefined,
    friction: undefined,
    gravity: true,
    fps: undefined,
    precision: undefined,

    // static numbers or functions that dynamically return what these
    // values should be for each element
    nodeMass: undefined, 
    edgeLength: undefined,

    stepSize: 1, // size of timestep in simulation

    // function that returns true if the system is stable to indicate
    // that the layout can be stopped
    stableEnergy: function( energy ){
      var e = energy; 
      return (e.max <= 0.5) || (e.mean <= 0.3);
    }
  };
  
  function ArborLayout(options){
    this.options = $$.util.extend({}, defaults, options);
  }
    
  ArborLayout.prototype.run = function(){
    var options = this.options;
    var cy = options.cy;
    var nodes = cy.nodes();
    var edges = cy.edges();
    var container = cy.container();
    var width = container.clientWidth;
    var height = container.clientHeight;
    var simulationBounds = options.simulationBounds;

    if( options.simulationBounds ){
      width = simulationBounds[2] -  simulationBounds[0]; // x2 - x1
      height = simulationBounds[3] - simulationBounds[1]; // y2 - y1
    } else {
      options.simulationBounds = [
        0,
        0, 
        width,
        height
      ];
    }

    // make nice x & y fields
    var simBB = options.simulationBounds;
    simBB.x1 = simBB[0];
    simBB.y1 = simBB[1];
    simBB.x2 = simBB[2];
    simBB.y2 = simBB[3];

    // arbor doesn't work with just 1 node 
    if( cy.nodes().size() <= 1 ){
      if( options.fit ){
        cy.reset();
      }

      cy.nodes().position({
        x: Math.round( (simBB.x1 + simBB.x2)/2 ),
        y: Math.round( (simBB.y1 + simBB.y2)/2 )
      });

      cy.one('layoutready', options.ready);
      cy.trigger('layoutready');

      cy.one('layoutstop', options.stop);
      cy.trigger('layoutstop');

      return;
    }

    var sys = this.system = arbor.ParticleSystem(options.repulsion, options.stiffness, options.friction, options.gravity, options.fps, options.dt, options.precision);
    this.system = sys;

    if( options.liveUpdate && options.fit ){
      cy.reset();
    };
    
    var doneTime = 250;
    var doneTimeout;
    
    var ready = false;
    
    var lastDraw = +new Date;
    var sysRenderer = {
      init: function(system){
      },
      redraw: function(){
        var energy = sys.energy();

        // if we're stable (according to the client), we're done
        if( options.stableEnergy != null && energy != null && energy.n > 0 && options.stableEnergy(energy) ){
          sys.stop();
          return;
        }

        clearTimeout(doneTimeout);
        doneTimeout = setTimeout(doneHandler, doneTime);
        
        var movedNodes = [];
        
        sys.eachNode(function(n, point){ 
          var id = n.name;
          var data = n.data;
          var node = data.element;
          
          if( node == null ){
            return;
          }
          
          if( !node.locked() && !node.grabbed() ){
            node.silentPosition({
              x: simBB.x1 + point.x,
              y: simBB.y1 + point.y
            });

            movedNodes.push( node );
          }
        });
        

        var timeToDraw = (+new Date - lastDraw) >= 16;
        if( options.liveUpdate && movedNodes.length > 0 && timeToDraw ){
          new $$.Collection(cy, movedNodes).rtrigger('position');
          lastDraw = +new Date;
        }

        
        if( !ready ){
          ready = true;
          cy.one('layoutready', options.ready);
          cy.trigger('layoutready');
        }
      }
      
    };
    sys.renderer = sysRenderer;
    sys.screenSize( width, height );
    sys.screenPadding( options.padding[0], options.padding[1], options.padding[2], options.padding[3] );
    sys.screenStep( options.stepSize );

    function calculateValueForElement(element, value){
      if( value == null ){
        return undefined;
      } else if( typeof value == typeof function(){} ){
        return value.apply(element, [element._private.data, {
          nodes: nodes.length,
          edges: edges.length,
          element: element
        }]); 
      } else {
        return value;
      }
    }
    
    // TODO we're using a hack; sys.toScreen should work :(
    function fromScreen(pos){
      var x = pos.x;
      var y = pos.y;
      var w = width;
      var h = height;
      
      var left = -2;
      var right = 2;
      var top = -2;
      var bottom = 2;
      
      var d = 4;
      
      return {
        x: x/w * d + left,
        y: y/h * d + right
      };
    }
    
    var grabHandler = function(e){
      var pos = sys.fromScreen( this.position() );
      var p = arbor.Point(pos.x, pos.y);
      this.scratch().arbor.p = p;
      
      switch( e.type ){
      case 'grab':
        this.scratch().arbor.fixed = true;
        break;
      case 'free':
        this.scratch().arbor.fixed = false;
        //this.scratch().arbor.tempMass = 1000;
        break;
      }
    };
    nodes.bind('grab drag free', grabHandler);
          
    nodes.each(function(i, node){
      if( this.isFullAutoParent() ){ return; } // they don't exist in the sim

      var id = this._private.data.id;
      var mass = calculateValueForElement(this, options.nodeMass);
      var locked = this._private.locked;

      if( node.isFullAutoParent() ){
        return;
      }
      
      var pos = fromScreen({
        x: node.position().x,
        y: node.position().y
      });

      if( node.locked() ){
        return;
      }

      this.scratch().arbor = sys.addNode(id, {
        element: this,
        mass: mass,
        fixed: locked,
        x: locked ? pos.x : undefined,
        y: locked ? pos.y : undefined
      });
    });
    
    edges.each(function(){
      var id = this.id();
      var src = this.source().id();
      var tgt = this.target().id();
      var length = calculateValueForElement(this, options.edgeLength);
      
      this.scratch().arbor = sys.addEdge(src, tgt, {
        length: length
      });
    });
    
    function packToCenter(callback){
      // TODO implement this for IE :(
      
      if( options.fit ){
        cy.fit();
      }
      callback();
    };
    
    var grabbableNodes = nodes.filter(":grabbable");
    // disable grabbing if so set
    if( options.ungrabifyWhileSimulating ){
      grabbableNodes.ungrabify();
    }
    
    var doneHandler = function(){
      if( window.isIE ){
        packToCenter(function(){
          done();
        });
      } else {
        done();
      }
      
      function done(){
        if( !options.liveUpdate ){
          if( options.fit ){
            cy.reset();
          }

          cy.nodes().rtrigger('position');
        }

        // unbind handlers
        nodes.unbind('grab drag free', grabHandler);
        
        // enable back grabbing if so set
        if( options.ungrabifyWhileSimulating ){
          grabbableNodes.grabify();
        }

        cy.one('layoutstop', options.stop);
        cy.trigger('layoutstop');
      }
    };
    
    sys.start();
    if( options.maxSimulationTime != null && options.maxSimulationTime > 0 && options.maxSimulationTime !== Infinity ){
      setTimeout(function(){
        sys.stop();
      }, options.maxSimulationTime);
    }
    
  };

  ArborLayout.prototype.stop = function(){
    if( this.system != null ){
      system.stop();
    }
  };
  
  $$('layout', 'arbor', ArborLayout);
  
  
})(cytoscape);
